@@ -1161,3 +1161,23 @@ def partition_dict(items, key):
     left = collections.OrderedDict(itertools.takewhile(unmatched, items_iter))
     right = collections.OrderedDict(items_iter)
     return left, item, right
+
+
+def ensure_unique(iterable, key=lambda x: x):
+    """
+    Wrap an iterable to raise a ValueError if non-unique values are encountered.
+
+    >>> consume(ensure_unique('abc'))
+    >>> consume(ensure_unique('abca'))
+    Traceback (most recent call last):
+    ...
+    ValueError: Duplicate element 'a' encountered.
+    """
+    seen = set()
+    seen_add = seen.add
+    for element in iterable:
+        k = key(element)
+        if k in seen:
+            raise ValueError(f"Duplicate element {element!r} encountered.")
+        seen_add(k)
+        yield element
